@@ -3,6 +3,6 @@
 reVX utilities.
 """
 from .exclusions_converter import ExclusionsConverter
-from .forecasts import Forecasts
+from .forecasts import FcstUtils
 from .output_extractor import output_extractor
 from .region_classifier import RegionClassifier
