@@ -3,4 +3,4 @@
 Compute prominent wind directions from PowerRose
 """
 from .mean_wind_dirs import MeanWindDirections
-from .prominent_wind_dirs import ProminentWindDirections
+from .prominent_wind_dirs import ProminentWindDirections, row_col_indices
